"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QuestQuery = void 0;
const schema_compiler_1 = require("@cubejs-backend/schema-compiler");
const GRANULARITY_TO_INTERVAL = {
    second: 's',
    minute: 'm',
    hour: 'h',
    day: 'd',
    month: 'M',
    year: 'Y'
};
class QuestParamAllocator extends schema_compiler_1.ParamAllocator {
    paramPlaceHolder(paramIndex) {
        return `$${paramIndex + 1}`;
    }
}
class QuestFilter extends schema_compiler_1.BaseFilter {
    orIsNullCheck(column, not) {
        return `${this.shouldAddOrIsNull(not) ? ` OR ${column} = NULL` : ''}`;
    }
    setWhere(column) {
        return `${column} != NULL`;
    }
    notSetWhere(column) {
        return `${column} = NULL`;
    }
}
class QuestQuery extends schema_compiler_1.BaseQuery {
    newFilter(filter) {
        return new QuestFilter(this, filter);
    }
    newParamAllocator() {
        return new QuestParamAllocator();
    }
    concatStringsSql(strings) {
        return `concat(${strings.join(', ')})`;
    }
    convertTz(field) {
        return `to_timezone(${field}, '${this.timezone}')`;
    }
    timeStampCast(value) {
        return value;
    }
    dateTimeCast(value) {
        return value;
    }
    subtractInterval(date, interval) {
        const [number, type] = this.parseInterval(interval);
        return `dateadd('${type}', ${-number}, ${date})`;
    }
    addInterval(date, interval) {
        const [number, type] = this.parseInterval(interval);
        return `dateadd('${type}', ${number}, ${date})`;
    }
    unixTimestampSql() {
        // QuestDB's now() function returns epoch timestamp with microsecond granularity.
        return 'now() / 1000000';
    }
    timeGroupedColumn(granularity, dimension) {
        const interval = GRANULARITY_TO_INTERVAL[granularity];
        if (interval === undefined) {
            throw new Error(`${granularity} granularity is not supported`);
        }
        return `timestamp_floor('${GRANULARITY_TO_INTERVAL[granularity]}', ${dimension})`;
    }
    dimensionsJoinCondition(leftAlias, rightAlias) {
        const dimensionAliases = this.dimensionAliasNames();
        if (!dimensionAliases.length) {
            return '1 = 1';
        }
        return dimensionAliases
            .map(alias => `(${leftAlias}.${alias} = ${rightAlias}.${alias} OR (${leftAlias}.${alias} = NULL AND ${rightAlias}.${alias} = NULL))`)
            .join(' AND ');
    }
    primaryKeyCount(cubeName, distinct) {
        if (distinct) {
            const primaryKeySql = this.primaryKeySql(this.cubeEvaluator.primaryKeys[cubeName], cubeName);
            return `count_distinct(${primaryKeySql})`;
        }
        else {
            return 'count(*)';
        }
    }
    orderHashToString(hash) {
        // QuestDB has partial support for order by index column, so map these to the alias names.
        // So, instead of:
        // SELECT col_a as "a", col_b as "b" FROM tab ORDER BY 2 ASC
        //
        // the query should be:
        // SELECT col_a as "a", col_b as "b" FROM tab ORDER BY "b" ASC
        if (!hash || !hash.id) {
            return null;
        }
        const fieldAlias = this.getFieldAlias(hash.id);
        if (fieldAlias === null) {
            return null;
        }
        const direction = hash.desc ? 'DESC' : 'ASC';
        return `${fieldAlias} ${direction}`;
    }
    getFieldAlias(id) {
        const equalIgnoreCase = (a, b) => (typeof a === 'string' && typeof b === 'string' && a.toUpperCase() === b.toUpperCase());
        let field;
        field = this.dimensionsForSelect().find((d) => equalIgnoreCase(d.dimension, id));
        if (field) {
            return field.aliasName();
        }
        field = this.measures.find((d) => equalIgnoreCase(d.measure, id) || equalIgnoreCase(d.expressionName, id));
        if (field) {
            return field.aliasName();
        }
        return null;
    }
    groupByClause() {
        // QuestDB doesn't support group by index column, so map these to the alias names.
        // So, instead of:
        // SELECT col_a as "a", count() as "c" FROM tab GROUP BY 1
        //
        // the query should be:
        // SELECT col_a as "a", count() as "c" FROM tab GROUP BY "a"
        if (this.ungrouped) {
            return '';
        }
        const names = this.dimensionAliasNames();
        return names.length ? ` GROUP BY ${names.join(', ')}` : '';
    }
}
exports.QuestQuery = QuestQuery;
//# sourceMappingURL=QuestQuery.js.map